! This soubroutine has one function (BEmax) which calculates, by backward induction,  the OLS parameters to interpolate EmaxV
! The inputs are ksi, xx, and loc.  


MODULE Emax_Betas


IMPLICIT NONE


CONTAINS

! ------------------------------------------------------------------------------------------------------------
FUNCTION BEmax(ksi, xx_giddp,  loc) 

! ------------------------------------------------------------------------------------------------------------
! It calculates, by backward induction,  the OLS parameters to interpolate EmaxV
! ------------------------------------------------------------------------------------------------------------
 
USE MATRIX
USE variables
USE OLS_interpolation

IMPLICIT  NONE

INTEGER, INTENT(IN)    :: ksi, loc
double precision, INTENT(IN) :: xx_giddp(dim_xx_dp)
double precision :: BEmax(dim_betas,bigS2,bigT-1)
    
integer:: Time

double precision, PARAMETER :: length_Ea=1.0d0, min_Ea=-0.6d0, length_Va=0.5d0, min_Va=0.05d0
double precision, PARAMETER :: length_test=6d0, min_test=-3d0, length_cut=6d0, min_cut=-4d0  
double precision, PARAMETER :: length_tec=50d0, min_tec=1d0, length_tui=5d0, min_tui=0.0d0 

! aux variables for loops
INTEGER :: i_grid, i_sum, i_s, i_s2, i_time ,iter_sch, ii_test

double precision :: Pr_PassAux
double precision :: xx(dim_x), EmaxT(SizeGrid,1), WW_aux(dim_W), Baux(dim_betas,1), Baux2(dim_SSpace+1,1)
double precision :: TV00,TV10,TV01,TV11,CV00,CV10,CV01,CV11,U00,U10,U01,U11,GPA0

double precision :: Test_grid, Teacher_grid, Cut_grid, Tuition_grid
INTEGER :: Jgrid(2), J_aux(2), W_inter(2), bigS_loc
INTEGER :: Ggrid, Egrid, Progrid, GstarGrid, GstarGrid2
double precision :: Sch_ww(dim_W),sums,sigma_aux, VecW1(bigS2), VecW2(bigS2) 
double precision :: temp,temp1,temp3,gpa_sim,Ea_grid,Va_grid,Cut_s,backC


double precision :: Xspace(SizeGrid,dim_SSpace), XIntT1(1,dim_SSpace)
Double precision :: updated_a(2),rho
integer :: next_t, tts
double precision, external :: terminalV,E_GPA,P_Pass,numerator_trans,util
double precision:: p_trans_vec(bigS), p_trans, p_trans_s
 
 
Time=bigT-1

if (xx_giddp(2)==1) then                   
  xx(3)=dble(0.5)*incomecut(1)
elseif (xx_giddp(2)==2) then
  xx(3)=dble(0.5)*(incomecut(1)+incomecut(2))
elseif (xx_giddp(2)==3) then
  xx(3)=dble(1.5)*incomecut(2)
endif

xx(1)=xx_giddp(1)

xx(2)=dble(0)
xx(4:dim_x)=dble(0)


!---------------------------------------------------------------------------------------------------------------------------
!                             Set the seed for random numbers
!---------------------------------------------------------------------------------------------------------------------------

CALL RANDOM_SEED(size = i_size)
CALL RANDOM_SEED(put = i_seed)
 
!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
!        -   solving the DP problem
!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------


!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
!        -   Decision at the begining of T (T==8). OUTPUT=BEmax(1:dim_betas,1:bigS2,7) 
!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
rho=R_arrest(loc) 
bigS_loc=N_sch_loc(loc)

VecW1(1:3)=0.0d-0; VecW1(4:6)=1;                            ! The two school characteristics that we do not interpolate
VecW2(1)=1; VecW2(2)=2; VecW2(3)=3;
VecW2(4)=1; VecW2(5)=2; VecW2(6)=3;

! We solve the problem for each school	
Do iter_sch=1,bigS2
	
    Sch_ww(1)=VecW1(iter_sch)
    Sch_ww(2)=VecW2(iter_sch)
    
    
    Xspace=0.0d-0

    DO i_grid=1,SizeGrid

        ! we simulate the grid of the state space
        Ea_grid=Aux_Ea_grid(i_grid,Time)*length_Ea+min_Ea;
        Va_grid=Aux_Va_grid(i_grid,Time)*length_Va+min_Va;
        Jgrid(1)=nint(Aux_Jgrid1(i_grid,Time)*Time);                 ! Total arrest
        if (Jgrid(1)==0) then
          Jgrid(2)=0 
        else	
          Jgrid(2)=nint(Aux_Jgrid2(i_grid,Time)*Time);         ! Total Time in Jail 
        endif
        
        Ggrid=min_G+nint(Aux_Ggrid(i_grid,Time)*Time);
        Progrid=int(Aux_Progrid(i_grid,Time)*dble(2));               !Academic Progress last period
        Egrid=int(Aux_Egrid(i_grid,Time)*dble(2));
        
        Test_grid=Aux_Test_grid(i_grid,Time)*length_test+min_test;          ! School Test score 
        Teacher_grid=Aux_Teacher_grid(i_grid,Time)*length_tec+min_tec;         ! Teacher quality
        Cut_grid=Aux_Cut_grid(i_grid,Time)*length_cut+min_cut;             ! Cut off for grade retention
        Tuition_grid=Aux_Tuition_grid(i_grid,Time)*length_tui+min_tui;         ! Tuition
        
        Sch_ww(3)=Test_grid
        Sch_ww(4)=Teacher_grid
        Sch_ww(5)=Cut_grid
        Sch_ww(6)=Tuition_grid
        Cut_s=Cut_grid
        
        ! fixed cost of coming back to school
        if(Egrid==0) then 
            backC=fixcost(ksi)
            if(Jgrid(1)>0) backC=backC+morefixcost
        else
            backC=0.0d-0
        endif

        ! We calculate TV00, TV10, TV01, TV11
        
        ! TV00
        TV00=delta*terminalV(ksi, xx, loc, Ea_grid, Va_grid,Jgrid, Ggrid,Sch_ww) 
        
        
        !TV01
        temp=0.0d-0
        !J_aux(1)=min(Jgrid(1)+1,maxArrest)
        J_aux(1)=Jgrid(1)+1
        do i_sum=0,maxJ            
            J_aux(2)=min(Jgrid(2)+i_sum,bigT)
            temp=temp+P_Jtime(Jgrid(1),i_sum,Time)*delta*terminalV(ksi, xx, loc, Ea_grid, Va_grid,J_aux, Ggrid, Sch_ww)    
        end do 

        TV01=(1.0d-0-Rho)*delta*terminalV(ksi, xx, loc, Ea_grid, Va_grid, Jgrid, Ggrid, Sch_ww)+Rho*temp  
        
        !TV10 and TV11
        ! simulate the integral by montecarlo method 
        temp1=0.0d-0
        GPA0=E_GPA(xx,ksi,Sch_ww,Ea_grid,Ggrid+1,Progrid)        
        temp3=GPA0-Ea_grid
        sigma_aux=(Va_grid+sigma_gpa**dble(2))**dble(0.5)
            
        do i_sum=1,Size_simul_M            
            gpa_sim=GPA0+sigma_aux*Aux_gpa_sim(i_sum,Time) !we generate normal numbers (given Ea_grid and Va_grid)

            Pr_PassAux=P_pass(Cut_s,gpa_sim,ksi,Ggrid+1,Sch_ww(1))
            
           call updateA(gpa_sim-temp3,Ea_grid,Va_grid,Updated_a)   !Updated_a(1) new EA, Updated_a(2) new VA

 
	
	
           temp1=temp1+(1.0d-0-Pr_PassAux)*terminalV(ksi, xx, loc,updated_a(1),updated_a(2),Jgrid,Ggrid,Sch_ww)+ &   
           Pr_PassAux*terminalV(ksi, xx, loc,updated_a(1),updated_a(2),Jgrid,Ggrid+1,Sch_ww)
           
            
           
        end do 
        temp1=temp1/dble(Size_simul_M)
        
        TV10=temp1*delta
        
        TV11=(1.0d-0-Rho)*temp1*delta+Rho*temp     

        !We calculate the utility for the 4 choices 
        U00=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,1,bigT)+TV00
        U01=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,2,bigT)+TV01
        U10=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,3,bigT)-backC+TV10
        U11=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,4,bigT)-backC+TV11

        EmaxT(i_grid,1)=EulerC+dlog(dexp(U00)+dexp(U01)+dexp(U10)+dexp(U11))
        
        GstarGrid=0
        if(Ggrid > Gstar-1) then 
            GstarGrid=1
        endif 
        
        Xspace(i_grid,1)=exp(Ea_grid); Xspace(i_grid,2)=Va_grid; Xspace(i_grid,3)=Jgrid(1); Xspace(i_grid,4)=Jgrid(2); 
        Xspace(i_grid,5)=Ggrid; Xspace(i_grid,6)=exp(Test_grid); Xspace(i_grid,7)=Teacher_grid; Xspace(i_grid,8)=Cut_grid;
        Xspace(i_grid,9)=GstarGrid; Xspace(i_grid,10)=Progrid;  Xspace(i_grid,11)=Egrid;          
    !i_grid is ended
    end DO


    
    !!! OLS regression (to interpolate)
    Baux=OLS_est(Xspace,EmaxT,SizeGrid)	
    BEmax(1:dim_betas,iter_sch,Time)=Baux(1:dim_betas,1)
    
end do


! end do (Do iter_sch=1,bigS2) to calculate BEmax for each school (discrete types, SES and DEP) 
!---------------------------------------------------------------------------------------------------------------------------


!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
!        -   Decision at the begining of T-1=7, T-2, T-3..., T-6 (T==8). OUTPUT=BEmax(1:dim_betas,1:bigS2,1:6)
! Time=7, then i_time=6,5,4,3,2,1 	
!---------------------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------------------
DO i_time=Time-1,1,-1                 ! Time is T-1
!print*, 'i_time', i_time    
    	
    Do iter_sch=1,bigS2
    

    
    
        Sch_ww(1)=VecW1(iter_sch)
        Sch_ww(2)=VecW2(iter_sch)
        
        Xspace=0.0d-0

        DO i_grid=1,SizeGrid

            ! we simulate the grid of the state space  
            Ea_grid=Aux_Ea_grid(i_grid,i_time)*length_Ea+min_Ea;
            Va_grid=Aux_Va_grid(i_grid,i_time)*length_Va+min_Va;
            Jgrid(1)=nint(Aux_Jgrid1(i_grid,i_time)*i_time);         ! Total arrest
            if (Jgrid(1)==0) then
             Jgrid(2)=0 
            else	
             Jgrid(2)=nint(Aux_Jgrid2(i_grid,i_time)*i_time);         ! Total Time in Jail 
            endif
            Ggrid=min_G+nint(Aux_Ggrid(i_grid,i_time)*i_time);
            Progrid=int(Aux_Progrid(i_grid,i_time)*dble(2));               !Academic Progress last period
            Egrid=int(Aux_Egrid(i_grid,i_time)*dble(2));

            Test_grid=Aux_Test_grid(i_grid,i_time)*length_test+min_test;          ! School Test score 
            Teacher_grid=Aux_Teacher_grid(i_grid,i_time)*length_tec+min_tec;      ! Teacher quality
            Cut_grid=Aux_Cut_grid(i_grid,i_time)*length_cut+min_cut;              ! Cut off for grade retention
            Tuition_grid=Aux_Tuition_grid(i_grid,i_time)*length_tui+min_tui;      ! Tuition
        
            Sch_ww(3)=Test_grid
            Sch_ww(4)=Teacher_grid
            Sch_ww(5)=Cut_grid
            Sch_ww(6)=Tuition_grid
            Cut_s=Cut_grid

            ! fixed cost of coming back to school
            if(Egrid==0) then 
                backC=fixcost(ksi)
                if(Jgrid(1)>0) backC=backC+morefixcost
            else
                backC=0.0d-0
            endif
            
            GstarGrid=0
            if(Ggrid > Gstar-1) then 
              GstarGrid=1
            endif 
     
            ! We calculate CV00, CV10, CV01, CV11 (by interpolation)

            ! Continuation payoff if (0,0)
            XIntT1=0
            XIntT1(1,1)=exp(Ea_grid); XIntT1(1,2)=Va_grid; XIntT1(1,3)=Jgrid(1); XIntT1(1,4)=Jgrid(2); XIntT1(1,5)=Ggrid; 
            XIntT1(1,6)=exp(Test_grid); XIntT1(1,7)=Teacher_grid; XIntT1(1,8)=Cut_grid; 
            XIntT1(1,9)=GstarGrid; XIntT1(1,10)=0.0d0; XIntT1(1,11)=0.0d0;  
             
        
            CV00=delta*INTERPOL(XIntT1,BEmax(1:dim_betas,iter_sch,i_time+1))

            
            ! Continuation payoff if (0,1)

            temp=0.0d-0
            J_aux(1)=Jgrid(1)+1 

            DO i_sum=0,maxJ            
                J_aux(2)=min(Jgrid(2)+i_sum,bigT)

                XIntT1=0
                XIntT1(1,1)=exp(Ea_grid); XIntT1(1,2)=Va_grid; XIntT1(1,3)=J_aux(1); XIntT1(1,4)=J_aux(2); XIntT1(1,5)=Ggrid; 
                XIntT1(1,6)=exp(Test_grid); XIntT1(1,7)=Teacher_grid; XIntT1(1,8)=Cut_grid; 
                XIntT1(1,9)=GstarGrid; XIntT1(1,10)=0.0d0; XIntT1(1,11)=0.0d0;  
                tts=nint(dble(i_sum)/dble(J_perYear)) !get the rounded number of years spent in jail.
                next_t=min(bigT-1,i_time+tts+1)

                temp=temp+P_Jtime(Jgrid(1),i_sum,Time)*delta**dble(tts+1)*INTERPOL(XIntT1,BEmax(1:dim_betas,iter_sch, &
                next_t))

            END DO 
            
            
            XIntT1=0
            XIntT1(1,1)=exp(Ea_grid); XIntT1(1,2)=Va_grid; XIntT1(1,3)=Jgrid(1); XIntT1(1,4)=Jgrid(2); XIntT1(1,5)=Ggrid; 
            XIntT1(1,6)=exp(Test_grid); XIntT1(1,7)=Teacher_grid; XIntT1(1,8)=Cut_grid; 
            XIntT1(1,9)=GstarGrid; XIntT1(1,10)=0.0d0; XIntT1(1,11)=0.0d0;       

            CV01= (1.0d-0-Rho)*delta*INTERPOL(XIntT1,BEmax(1:dim_betas,iter_sch,i_time+1))+Rho*temp 


  
            ! Continuation payoff if (1,0) and (1,1)

            ! simulate the integral by montecarlo method 
            temp1=0.0d-0
            GPA0=E_GPA(xx,ksi,Sch_ww,Ea_grid,Ggrid+1,Progrid)
            temp3=GPA0-Ea_grid
            
            sigma_aux=(Va_grid+sigma_gpa**dble(2))**dble(0.5)            
            DO i_sum=1,Size_simul_M            
                gpa_sim=GPA0+sigma_aux*Aux_gpa_sim(i_sum,i_time)                  !we generate normal numbers (given Ea_grid and Va_grid)
                Pr_PassAux=P_pass(Cut_s,gpa_sim,ksi,Ggrid+1,Sch_ww(1))
                
                call updateA(gpa_sim-temp3,Ea_grid,Va_grid,Updated_a)   !Updated_a(1) new EA, Updated_a(2) new VA
                
                
                XIntT1=0
                XIntT1(1,1)=exp(Updated_a(1)); XIntT1(1,2)=Updated_a(2); XIntT1(1,3)=Jgrid(1); XIntT1(1,4)=Jgrid(2); 
                XIntT1(1,5)=Ggrid; XIntT1(1,6)=exp(Test_grid); XIntT1(1,7)=Teacher_grid; XIntT1(1,8)=Cut_grid; 
                XIntT1(1,9)=GstarGrid; XIntT1(1,10)=0.0d0; XIntT1(1,11)=1.0d0;   
                
	
                temp1=temp1+(1.0d-0-Pr_PassAux)*INTERPOL(XIntT1,BEmax(1:dim_betas,iter_sch,i_time+1))
                	

                if (Ggrid/=Gstar-1) then
                    GstarGrid2=0
                       if(Ggrid + 1 > Gstar-1) then 
                         GstarGrid2=1
                       endif 
                    XIntT1=0
                    XIntT1(1,1)=exp(Updated_a(1)); XIntT1(1,2)=Updated_a(2); XIntT1(1,3)=Jgrid(1); XIntT1(1,4)=Jgrid(2); 
                    XIntT1(1,5)=Ggrid+1; XIntT1(1,6)=exp(Test_grid); XIntT1(1,7)=Teacher_grid; XIntT1(1,8)=Cut_grid; 
                    XIntT1(1,9)=GstarGrid2; XIntT1(1,10)=1.0d0; XIntT1(1,11)=1.0d0;
                    

                    temp1=temp1+Pr_PassAux*INTERPOL(XIntT1,BEmax(1:dim_betas,iter_sch,i_time+1))

                    
                else
                    
                	GstarGrid2=0
                      if(Ggrid + 1 > Gstar-1) then 
                        GstarGrid2=1
                      endif                 
                	
                    XIntT1=0
                    XIntT1(1,1)=exp(Updated_a(1)); XIntT1(1,2)=Updated_a(2); XIntT1(1,3)=Jgrid(1); XIntT1(1,4)=Jgrid(2); 
                    XIntT1(1,5)=Ggrid+1; XIntT1(1,9)=GstarGrid2; XIntT1(1,10)=1.0d0; XIntT1(1,11)=1.0d0;     
                    
                    p_trans_vec=dble(0)
                    DO i_s=1,bigS_loc
                        p_trans_vec(i_s)=numerator_trans(xx,Sch_ww,W(7:12,Mat_sch_loc(i_s,loc)),gpa_sim,ksi)
                    END DO
                    p_trans_s=sum(p_trans_vec(1:bigS_loc),dim=1)

                    
                    sums=0.0d-0
                    DO i_s=1,bigS_loc
                        p_trans=p_trans_vec(i_s)/p_trans_s
                        
                        W_inter(1:2)=nint(W(7:8,Mat_sch_loc(i_s,loc)))
                        
                        i_s2=1   
                        if (W_inter(1)==0 .and. W_inter(2)==2) then
                          i_s2=2
                        elseif (W_inter(1)==0 .and. W_inter(2)==3) then
                          i_s2=3  
                        elseif (W_inter(1)==1 .and. W_inter(2)==1) then
                          i_s2=4
                        elseif (W_inter(1)==1 .and. W_inter(2)==2) then
                          i_s2=5 
                        elseif (W_inter(1)==1 .and. W_inter(2)==3) then
                          i_s2=6    
                        endif
                        
                        XIntT1(1,6)=exp(W(9,Mat_sch_loc(i_s,loc))); XIntT1(1,7)=W(10,Mat_sch_loc(i_s,loc)); 
                        XIntT1(1,8)=W(11,Mat_sch_loc(i_s,loc)); 
                                             

                        sums=sums+p_trans*INTERPOL(XIntT1,BEmax(1:dim_betas,i_s2,i_time+1))     ! non crime and crime

                    END DO
                           
                    temp1=temp1+Pr_PassAux*sums         

                endif
                

            !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!           
            END DO 

              
            temp1=temp1/dble(Size_simul_M)

             CV10=temp1*delta
             
             CV11=(1.0d-0-Rho)*delta*temp1+Rho*temp   
              
             !We calculate the utility for the 4 choices 
             U00=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,1,i_time+1)+CV00
             U01=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,2,i_time+1)+CV01
             U10=util(xx,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,3,i_time+1)-backC+CV10
             U11=util(xX,ksi,Ea_grid,Sch_ww,Progrid,Loc,Jgrid,Ggrid,4,i_time+1)-backC+CV11
              
             EmaxT(i_grid,1)=EulerC+dlog(dexp(U00)+dexp(U01)+dexp(U10)+dexp(U11))

             Xspace(i_grid,1)=exp(Ea_grid); Xspace(i_grid,2)=Va_grid; Xspace(i_grid,3)=Jgrid(1); Xspace(i_grid,4)=Jgrid(2); 
             Xspace(i_grid,5)=Ggrid; Xspace(i_grid,6)=exp(Test_grid); Xspace(i_grid,7)=Teacher_grid; Xspace(i_grid,8)=Cut_grid;
             Xspace(i_grid,9)=GstarGrid; Xspace(i_grid,10)=Progrid;  Xspace(i_grid,11)=Egrid;                       
  
             !i_grid is ended
        END DO
   
        


        !!! OLS regression (to interpolate)
         if (i_time == 1) then
        	Baux=OLS_est1(Xspace,EmaxT,SizeGrid)
         elseif (i_time == 2 .or. i_time == 3) then
        	Baux=OLS_est23(Xspace,EmaxT,SizeGrid)
         elseif (i_time == 4) then
        	Baux=OLS_est4(Xspace,EmaxT,SizeGrid)
         else
         	Baux=OLS_est(Xspace,EmaxT,SizeGrid)
         endif
    
         BEmax(1:dim_betas,iter_sch,i_time)=Baux(1:dim_betas,1)
         


           
    END DO 
     ! end do (Do iter_sch=1,bigS2) to calculate BEmax for each school 
    

 END DO
 ! end do (Do i_time=Time,1,-1) to calculate BEmax for each period 
    	
	 
 !---------------------------------------------------------------------------------------------------------------------------
    	 
 !---------------------------------------------------------------------------------------------------------------------------

END FUNCTION BEmax
END MODULE EMax_Betas


