
!*********** Utility related***********
    double precision function util(xx,ksi,abi, ww,progress,loc, jails,bigG, yy,tt) 
        !expected current utility function for each choice
        !1. this is without the taste shocks, 2. expectation over GPA viewed by student given her belief about ability
        use variables
        implicit none
        integer,intent(in):: ksi !type
        integer,intent(in):: progress !=1 if normal progress, =0 if retained 
        integer,intent(in):: loc !location of the person, which affects criminal return
        integer,intent(in):: yy ! 1-4 four choices (e0,d0), (e0,d1), (e1,d0), (e1,d1)
        integer,intent(in):: tt !time/age
        double precision,intent(in):: xx(dim_X),ww(dim_W)
        integer,intent(in):: jails(2) !arrest history, jail history
        integer,intent(in):: bigG !grade completed (so in grade 9th, this is equal to 8)
        double precision:: price, SS, temp
        double precision, intent(in):: abi !ability viewed by the student (this is how he forms his expectation about his GPA)
        double precision, external :: costv, E_GPA,Ucrime, Norm_cdf
        
        
        price=ww(dim_W)
        SS=1.0d-0	
        if(bigG<8) SS=0.0d-0
        
        if (yy==1) then
            util=0.0d-0
        elseif(yy==2) then
            util=Ucrime(loc, ksi, jails,tt,xx,1)
        else
            temp=E_GPA(xx,ksi,ww(1:dim_W-2),abi,bigG+1,progress)
            if(yy==3) then
                util=paraC(7)+(lambda+lambdaSS*SS)*temp-costv(xx,ww,ksi,price,progress,0,bigG)
            elseif(yy==4) then
                util=paraC(7)+(lambda+lambdaSS*SS)*temp-costv(xx,ww,ksi,price,progress,1,bigG)+Ucrime(loc, ksi, jails,tt,xx,0)  
            endif  
            temp=(lowgpa-temp)/sigma_gpa
            util=util-lowGPAcost*Norm_cdf(temp,dble(0),dble(1))
        endif  
        if(yy>2.and.jails(1)>=1) util=util-extracost
           
    end function util
    
    
    double precision function costv(xx,ww,ksi,price,progress,crime,bigG)   !cost function for going to school TBD
        use variables 
        implicit none
        integer, intent (in):: ksi, progress,crime,bigG
        double precision,intent(in)::xx(dim_X),ww(dim_W), price
        integer:: n
        double precision::temp, Bigger
        
        Bigger=1	
        if(bigG<8) Bigger=0	
        
        temp=paraC(0)*price                      ! paraC(low income famility)=ParaC(0)
        if(xx(3)<incomecut(2) .and. xx(3)>=incomecut(1)) then
            temp=(1-paraC(1))*price    !paraC(mid income people)=1-ParaC(1)            
        elseif (xx(3)>=incomecut(2)) then
            temp=(1-paraC(2))*price    !paraC(high income people)=1-ParaC(2)
        endif
        
        temp=temp+dble(1-progress)*paraC(3)+(1-Bigger)*dble(crime)*paraC(4)+Bigger*(dble(crime)*paraC(5)+paraC(8))
        
        if(ksi==2) temp=temp+paraC(6)
        
        temp=temp-(1.0d-0-ww(1))*paraC(9)           !positive utility from attending private school
        if(ww(2)==1) temp=temp+(1-Bigger)*paraC(10)+Bigger*paraC(13)             !extra disutility for low SES school (presumably these schools have bad facilities)
        if(ww(2)==2) temp=temp+(1-Bigger)*paraC(11)+Bigger*paraC(14)             !extra disutility for Medium SES school (presumably these schools have bad facilities)
        if(ww(3)<-.35 .and. Bigger==0) temp=temp+paraC(12) ! extra disutilty for attending school bellow percentil 25 of test scores (PS)                  
        if(ww(3)<-.72 .and. Bigger==1) temp=temp+paraC(15) ! extra disutilty for attending school bellow percentil 25 of test scores (SS)
        
        if(ww(2)==int(xx(1)) .and. Bigger==1) temp=temp-paraC(16)               !extra utility for SES school= own SES
        
        costv=temp

    end function costv
    
    double precision function Ucrime(loc,ksi,jails,tt,xx,DropOut)
        use variables 
        implicit none
        integer, intent (in):: loc,ksi,jails(2),tt,DropOut
        double precision,intent(in):: xx(dim_x)
        double precision:: rho,temp,ss,crimepay,temp2
        integer:: t
        
        rho=R_arrest(loc)
        temp=0.0d-0
        do t=0,maxJ !utility in jail intergrated over months 
            ss=Ujail(0)+Ujail(1)*dble(t)+Ujail(2)*dble(t)**dble(2) !utility in jail: quadratic in jail time
            if(jails(1)<1) ss=ss+Ujail(3)
            temp=temp+P_Jtime(jails(1),t,tt)*ss
        enddo   
                       
        !specify the crime pay: depend on total crime rate and meean income in that location
        crimepay=paraCrimePay(0)+paraCrimePay(1)*(R_crime(1,loc)+R_crime(2,loc))+paraCrimePay(2)*MeanInc(loc)*1.0E-5
        
        if(tt>4) then
        temp2=paraCrimePay(5)
        endif
        
        if(int(xx(1))==1) then
        crimepay=crimepay+paraCrimePay(3)*MeanInc(loc)*1.0E-5
        endif 
        if(int(xx(1))==2) then
        crimepay=crimepay+paraCrimePay(4)*MeanInc(loc)*1.0E-5
        endif
        
        Ucrime=paraCrime0(ksi)+(1.0d-0-rho)*crimepay+rho*temp+temp2
        
    end function Ucrime
!****************************

!****** Performance Related******************

!NOTE: E_GPA: 
!1. when solve student's problem, the input "abi" in E_GPA is student's belief about his ability, the error term has mean zero (ability prediction error+gpa shock)
!2.when simulate student's GPA, the input "abi" in E_GPA is the ability level that we assign to the student, given his initial conditions and his type and the std test score observed only by us. 
!   the error term has mean zero (gpa shock) No ability prediction error, because there is no predcition. we already assign an ability level to the student
!               
    double precision function E_GPA(xx,ksi,ww,abi,g,progress) 
        use variables 
        implicit none
        integer, intent (in):: ksi,g,progress !g: grade
        double precision,intent(in):: xx(dim_X),ww(dim_W-2),abi
        integer:: n1
        double precision:: temp
        
        temp=0.0d-0

        !households
        n1=int(xx(1))
        !parental education x1: default group is the middle education group, para_gpa(1) for low, para_gpa(2) for high edu, 
        if(n1==1) then
            temp=temp+para_gpa(1)+ww(3)*para_gpa(11)       
        elseif(n1==3) then
            temp=temp+para_gpa(2)+ww(3)*para_gpa(12)      
        endif
  
        temp=temp+para_gpa(3)*xx(2) !single parent (para_gpa(3)==0 for now)
         
        temp=temp+para_gpa(4)*xx(6) !pre-schooled (para_gpa(4)==0 for now)
     
        
        !school
        temp=temp+para_gpa(5)*ww(1) !public school
      
        
        n1=int(ww(2)) 
        if(n1<3) then  !school SES, the highest type (3) is default group. 
            temp=temp+para_gpa(5+n1)
        endif

        if(xx(3)<incomecut(1)) then
            temp=temp+para_gpa(8)  
        elseif(xx(3)<incomecut(2)) then
            temp=temp+para_gpa(9) 
        endif      
        
        temp=temp+ww(3)*para_gpa(10)+ww(4)*para_gpa(13)
     
        
        
        E_GPA=para_gpa0(g-4)+temp+abi+para_gpa(dim_paraGPA)*dble(1-progress)

    end function E_GPA
    
    double precision function E_S0(xx, abi) 
        use variables 
        implicit none
        double precision,intent(in):: xx(dim_X),abi
        integer:: n1
        double precision:: temp
        
        temp=0.0d-0

        !households
        n1=int(xx(1))
        !parental education x1: default group is the middle education group, para_gpa(1) for low, para_gpa(2) for high edu, 
        if(n1==1) then
            temp=temp+para_gpa(1)+ww(3)*para_gpa(11)       
        elseif(n1==3) then
            temp=temp+para_gpa(2)+ww(3)*para_gpa(12)      
        endif
  
        temp=temp+para_gpa(3)*xx(2) !single parent (para_gpa(3)==0 for now)
         
        temp=temp+para_gpa(4)*xx(6) !pre-schooled (para_gpa(4)==0 for now)
     
        
        !school
        temp=temp+para_gpa(5)*ww(1) !public school
      
        
        n1=int(ww(2)) 
        if(n1<3) then  !school SES, the highest type (3) is default group. 
            temp=temp+para_gpa(5+n1)
        endif

        if(xx(3)<incomecut(1)) then
            temp=temp+para_gpa(8)  
        elseif(xx(3)<incomecut(2)) then
            temp=temp+para_gpa(9) 
        endif      
        
        temp=temp+ww(3)*para_gpa(10)+ww(4)*para_gpa(13) 
                
     
        E_S0=paraS(0)+temp+paraS(1)*abi
         
    end function E_S0


    double precision function P_pass(cut,gpa,ksi,g,Spublic)
!        !DEC$ OBJCOMMENT LIB:'libiomp5md.lib'
!        !USE IMSL_LIBRARIES
        USE OMP_LIB
        use variables
        implicit none
        double precision,intent(in):: cut, gpa, Spublic
        integer,intent(in)::ksi, g !g: grade
        double precision:: temp

        double precision, external :: Norm_cdf
        
        
        if(g>8) then
           temp=(gpa+para_pass(ksi)+cut_public(2)*Spublic-cut-cut_high(1))/sigma_pass   
        else 
           temp=(gpa+para_pass(ksi)+cut_public(1)*Spublic-cut-cut_prim)/sigma_pass   
        endif
        
        P_pass=Norm_cdf(temp,dble(0),dble(1))

    end function P_Pass
!***************

!********* Transfer schools*********** 
    double precision function numerator_trans(xx,ww0,ww1,gpa,ksi)
        use variables               !! ww0 current school characteristics
        implicit none               !! ww1 new school characteristics
        integer,intent(in):: ksi
        double precision,intent(in)::xx(dim_X),ww0(dim_w),ww1(dim_w),gpa
        double precision:: temp         
        
        temp=0.0d-0
        
        if (ww0(3)-Medw3_5>0) then
           temp=temp+(ww0(3)-Medw3_5)*(ww1(3)-Medw3_10)*para_trans(1)
        else
           temp=temp+(ww0(3)-Medw3_5)*(ww1(3)-Medw3_10)*para_trans(2)	   
        endif    
     
        
         if (ww0(6)-ww1(6)>0) then
          temp=temp+(ww0(6)-ww1(6))**dble(2)*para_trans(3)
        elseif (ww0(6)-ww1(6)< 0) then
           temp=temp+(ww0(6)-ww1(6))**dble(2)*para_trans(4)    
        endif 

        if (gpa-Medgpa>0) then
           temp=temp+(gpa-Medgpa)*(ww1(3)-Medw3_10)*para_trans(5)
        else
           temp=temp+(gpa-Medgpa)*(ww1(3)-Medw3_10)*para_trans(6)	   
        endif 
        
        temp=temp+(1-ww1(1))*para_trans(7)+(1-ww0(1))*(1-ww1(1))*para_trans(8)
        
        if (int(ww0(2))==1 .AND. int(ww1(2))==1) temp=temp+para_trans(9)
        if (int(ww0(2))==1 .AND. int(ww1(2))==3) temp=temp+para_trans(10)
        if (int(ww0(2))==2 .AND. int(ww1(2))==1) temp=temp+para_trans(11)
        if (int(ww0(2))==2 .AND. int(ww1(2))==3) temp=temp+para_trans(12)
        if (int(ww0(2))==3 .AND. int(ww1(2))==1) temp=temp+para_trans(13)
        if (int(ww0(2))==3 .AND. int(ww1(2))==3) temp=temp+para_trans(14)

        if (int(xx(1))==1 .AND. int(ww1(2))==1) temp=temp+para_trans(15)
        if (int(xx(1))==1 .AND. int(ww1(2))==3) temp=temp+para_trans(16)
        if (int(xx(1))==3 .AND. int(ww1(2))==1) temp=temp+para_trans(17)
        if (int(xx(1))==3 .AND. int(ww1(2))==3) temp=temp+para_trans(18)

        
        if(ksi==2) temp=temp+(1-ww1(1))*para_trans(21)	
        if (ww0(3)-Medw3_5<=0 .AND. ksi==2) temp=temp+(ww0(3)-Medw3_5)*(ww1(3)-Medw3_10)*para_trans(22)	       
        
        numerator_trans=dexp(temp)

    end function numerator_trans
    

!*********************    

!***********Belief Updating 
    !***Update using GPA***********
    subroutine updateA(yy,EA, VA,news)
        use variables
        implicit none
        double precision,intent(in):: yy,EA, VA
        double precision,intent(out):: news(2)
        double precision:: temp
        temp=VA/(sigma_gpa**dble(2)+VA)
        news(1)=EA*(1.0d-0-temp)+yy*temp        
        news(2)=temp*sigma_gpa**dble(2)
    end subroutine updateA

    !***Update using additional info available only to the researcher***********
 
    subroutine updateA_addition(yy,EA, VA,news)
        use variables
        implicit none
        double precision,intent(in):: yy,EA, VA
        double precision,intent(out):: news(2)
        double precision:: temp
        temp=VA/(sigma_s0**dble(2)+VA)
        news(1)=EA*(1.0d-0-temp)+yy*temp        
        news(2)=temp*sigma_s0**dble(2)
    end subroutine updateA_addition
   
!******************
        

!**********TYPE************
    subroutine Ptype(xx,ww,ps) 
        use variables
        implicit none
        double precision,intent(in):: xx(dim_X), ww(dim_w)
        double precision,intent(out):: ps(Ntype-1)
        integer:: n1,n
        double precision:: temp
        do n=1,Ntype-1
            temp=paraP(0,n)
            n1=int(xx(1))
            if(n1<3) then
                temp=temp+paraP(n1,n)        !parental education x1: default group is the highest education group, para_gpa(1) for low, para_gpa(2) for middle edu, 
            endif
            temp=temp+paraP(3,n)*xx(2) !participation in social programs
            temp=temp+paraP(4,n)*xx(3)*1.0E-5 !income
            temp=temp+paraP(5,n)*xx(4) !private insurance
            n1=int(xx(5))
            if(n1<3) then
                temp=temp+paraP(n1+5,n)        !parental contract, the highest type (3) is default 
            endif
            temp=temp+paraP(8,n)*xx(6) !pre-schooled
            if(int(xx(7))>4) then   !big family
                temp=temp+paraP(9,n)
            endif
            temp=temp+paraP(10,n)*ww(3)+paraP(11,n)*(1-ww(1))+paraP(12,n)*ww(6)
            
         
            ps(n)=dexp(temp)/(1.0d-0+dexp(temp))
        enddo           
    end subroutine Ptype
!***********************

!*********** Terminal V*************** 
    double precision function terminalV(ksi, xx, loc, Ea, Va,  bigJ, bigG,ww)
        use variables
        implicit none
        integer, intent(in):: ksi, loc,  bigJ(2), bigG !(time period, type, s, location, crime history, arrest and jail history, grade completed, school id)
        double precision,intent(in)::ww(dim_W), Ea, Va, xx(dim_X) !beliefs about ability (mean, variance)
        double precision:: temp, Bigger
        integer:: n1
        
        Bigger=1	
        if(bigG<8) Bigger=0	
        
        temp=para_terminal(1)*bigG+para_terminal(5)*exp(Ea)*bigG+para_terminal(6)*exp(ww(3))*bigG*Bigger 
        temp=temp+para_terminal(16)*dble(bigJ(1))+para_terminal(17)*dble(bigJ(2))
        
         
        if(ksi==1) then
           temp=temp+para_terminal(3)*dble(bigJ(1))+para_terminal(4)*dble(bigJ(2))+para_terminal(7)*bigG	   
        endif     
        
        
        n1=int(xx(1))
        if(n1==2) then
           temp=temp+para_terminal(8)*bigG
           if(bigG==12) temp=temp+para_terminal(10)
        elseif(n1==3) then
           temp=temp+para_terminal(9)*bigG
           if(bigG==12) temp=temp+para_terminal(11)
        endif         

        if(bigG==12) then
           temp=temp+para_terminal(12)+exp(Ea)*para_terminal(13)+exp(ww(3))*para_terminal(2)
           if(ksi==2) temp=temp+para_terminal(15)
        endif 
        
        if(bigJ(1)>0) then
           temp=temp+para_terminal(14)
        endif 
        
        terminalV=temp
    end function terminalV
!*******************

!********* Normal PDF ********
    double precision function p_normal(b,mu,sigmas)
        use variables
        implicit none
        double precision, intent(in):: b, mu,sigmas
        double precision:: temp
        temp=(b-mu)/sigmas
        
        p_normal=dexp(-temp**dble(2)/dble(2))/sigmas/(2*ppi)**dble(0.5)
        
    end function p_normal
!***************


!********* Normal CDF********
REAL(8) FUNCTION Norm_cdf(zin,mu,var)
IMPLICIT NONE
REAL(8), INTENT(IN) :: zin
REAL(8), INTENT(IN) :: mu,var
REAL(8), PARAMETER :: small=1.0D-250
REAL(8) :: zabs,p,arg,logpdf,z,std
REAL(8), PARAMETER :: p0=220.2068679123761D0,p1=221.2135961699311D0,p2=112.0792914978709D0, &
	p3 = 33.91286607838300D0,p4 = 6.373962203531650D0,p5 = .7003830644436881D0, &
    p6 = .3526249659989109D-01,q0 = 440.4137358247522D0,q1 = 793.8265125199484D0, &
    q2 = 637.3336333788311D0,q3 = 296.5642487796737D0,q4 = 86.78073220294608D0, &
    q5=16.06417757920695D0,q6=1.755667163182642D0,q7=.8838834764831844D-1,cutoff = 7.071D0, &
	logroot2pi = 0.918938533204672780563271317078D0
IF (var<0.0d0) THEN
	PAUSE '***FATAL ERROR: variance has to be positive (Norm_cdf)***'
    STOP
END IF    
z=zin-mu
zabs=ABS(z)  
IF (zabs<small) THEN
	Norm_cdf=0.5d0
	RETURN
END IF
std = DSQRT(var)
IF (std<small) THEN
	IF (zin-mu>0.0d0) THEN
		Norm_cdf = 1.0d0
	ELSE IF	(zin-mu<0.0d0) THEN
		Norm_cdf = 0.0d0
	END IF
END IF
zabs=zabs/std
IF (z > 37.0D0) THEN
	Norm_cdf = 1.0D0
	RETURN
ELSE IF (z < -37.0D0) THEN
	Norm_cdf = 0.0D0
	RETURN
END IF
arg = -0.5D0*zabs*zabs
logpdf = -logroot2pi - DLOG(std) + arg
IF (zabs < cutoff) THEN
	p = arg + DLOG(((((((p6*zabs + p5)*zabs + p4)*zabs + p3)*zabs + &
	p2)*zabs + p1)*zabs + p0)) - DLOG((((((((q7*zabs + q6)*zabs + &
	q5)*zabs + q4)*zabs + q3)*zabs + q2)*zabs + q1)*zabs + &
	q0))
ELSE
	p = logpdf - DLOG((zabs + 1.0D0/(zabs + 2.0D0/(zabs + 3.0D0/(zabs + 4.0D0/ &
	(zabs + 0.65D0))))))
END IF
p = DEXP(p)
IF (z < 0.0D0) THEN
	Norm_cdf=p
	RETURN
ELSE
	Norm_cdf = 1.0D0 - p
	RETURN
END IF
RETURN
END FUNCTION Norm_cdf
!***************   
        
        

