module variables
implicit none

! Instruct
    integer:: purpose !=1 if estimate, =2  if simulate baseline
    integer:: Writedetail,iseed !=1 if writeout detailed simulation result, random number generator seed
    integer:: std_err !=0 if don't calculate stderr, =1 if calculate the likelihood at original param, =2 if at shifted param
    double precision:: incomecut(2) !cutoffs of income that divide population into poor, middle, rich
    integer:: whichcounter !=1 if change transfer prob, =2 if change inital school
    double precision:: medianInc  ! median income, it is used in the type distribution function. 
    double precision:: lowGPA   !the cutoff for low GPA
! Dimension (TBD)
    INTEGER, PARAMETER :: SizeGrid=500, Size_simul_M=50 ! size interpolation, size montecarlo grid 
    integer,parameter::  dataN=34783, copies=50, bigR=50 ! num of obs., replica in simulation, in estimation
    integer,parameter:: dim_X=7,dim_W=6 !dimension of household x, school characteristics W
    integer,parameter:: Nloc=34   !number of locations/municipalities
    integer,parameter:: dim_xx_dp=2, SizeDimXX=9   !number of dimension of xx considered in DP, number of xx types (that enter in the dynnamic problem)/vector of types
    integer, parameter:: bigT=8   !time horizon
    integer,parameter:: Ntype=2 !num of types
    integer,parameter:: dim_betas=75 !dimension of the OLS interpolation for the EMax
    integer,parameter:: dim_SSpace=11 !dimension of the State space
    integer,parameter:: dim_paraU=25 !dimension of utility param in school
    integer,parameter:: dim_paraP=12 !dimension of type param
    integer,parameter:: dim_paraC=16 !dimension of cost function (it starts in 0)
    integer,parameter:: dim_paraGPA=14 !dimension of GPA production funciton (12+Ntype)
    integer,parameter:: dim_paraTrans=22 !school transfer prob
    integer,parameter:: dim_paraCrime=5 !crime payoff for each location
    integer,parameter:: dim_Ujail=3 !utility in jail
    integer,parameter:: dim_ParaTerminal=17 ! parameters terminal value
    integer,parameter:: dim_oldGPA=4 !dimension of gpa before the initial period of the model
    integer,parameter:: bigS=3388 !all schools
    integer,parameter:: bigS2=6  ! All combinations of W1 and W2
    integer,parameter:: Gstar=8 !transition period of school
    integer,parameter:: dim_cuth=1
    integer,parameter:: J_perYear=2 !how many jail spells are there in each year. (we measure jail time in unit of 12/J_perYear)
    integer,parameter:: maxJ=11   !maximum length of jail time (measured in jailmonths) sentenced (maximum 5 years* J_perYear+ >5 years, but zero is the first category)
    integer,parameter:: maxArrest=8
    integer,parameter:: N_GPA=20 !# grid points in GPA
    integer,parameter:: N_schType=10 !# schools in each discretized category
    double precision, parameter:: ppi=3.1415926535
    INTEGER, PARAMETER :: min_G=4
    double precision, PARAMETER :: EulerC=0.5772156649
    INTEGER, PARAMETER :: Max_num_sch=393 ! The max size of (municipality) choice set 
    
!Data: Households
    double precision:: delta=0.9d-0 !discount rate
    double precision,allocatable:: X(:,:)   !household characteristics (dimension of x, household id)
    double precision,allocatable:: X_girdDP(:,:) !household characteristics (grid of points for the dynamic proble)
    double precision, allocatable:: s0(:) !initial test score 
    double precision,allocatable:: oldGPA(:,:)
    integer,allocatable:: sch0(:) ! which school the student is at initially
    integer,allocatable:: hh_loc(:)
    double precision, allocatable:: P_ksi(:,:) !the prob of types for each student P_ksi(type,student id)
    integer,allocatable:: id_xx(:)
    double precision,allocatable:: dGPA(:,:)
    integer,allocatable:: dbigG(:,:),dArrests(:,:), dJails(:,:), dNewschool(:),dEnroll(:,:),dinactive(:,:)
    

!Data: School
    double precision:: W(dim_W+dim_W,bigS)
    integer:: offerHigh(bigS) !=1 if school offers high grades
    integer:: Missing_W(bigS) ! =1 w_05 is missing, =2 w_10 is missing, =3 no information is missing 
    
!Data: location
    double precision:: R_arrest(Nloc)
    double precision:: R_crime(2,Nloc) !crime rate for younger and older teenagers.
    double precision:: MeanInc(Nloc) !Average income in location
    double precision:: Frac_goodSchool(Nloc) 
    double precision:: P_Jtime(0:maxArrest,0:maxJ,bigT) !P_Jtime(n1,n3,t): prob of sentenced to n3 years in prison given a history of n1 arrests and age=t .
    integer:: N_sch_loc(Nloc)   ! to map with the School ID 
    integer:: Mat_sch_loc(Max_num_sch,Nloc)   ! to map with the School ID
    
! Model parameters
    double precision:: paraC(0:dim_paraC)   !utility cost parameter of attending school
    double precision:: fixcost(2),morefixcost    !fixed cost of going back to school if one was not enrolled last period, one for each type 
    double precision:: extracost    !extra cost of being in school for someone with arrest history. 
    double precision:: Ujail(0:dim_Ujail) !utility in jail (should be negative)
    double precision:: paraCrimePay(0:dim_paraCrime) !payoff of crime uncaught
    double precision:: paraCrime0(Ntype) !utility of committing crime type specific
    double precision:: lambda        ! utility weight on GPA
    double precision:: lambdaSS              ! utility weight on GPA (Secondary School)
    double precision:: paraP(0:dim_paraP,Ntype-1)   !parameter governing type distribution
    double precision:: sigma_s0 !std dev of initial test score
    double precision:: sigma_gpa ,sigma_pass !std dev of gpa, passing a grade
    double precision:: para_Pass(Ntype)   !type-specific parameter in the passing function.
    double precision:: meanA(Ntype),sigmaA(Ntype) !mean and std err of ability conditional on type
    double precision:: para_gpa(dim_paraGPA)
    double precision:: para_gpa0(bigT)    !grade-specific intercept
    double precision:: para_trans(dim_paraTrans)
    double precision:: para_terminal(dim_paraTerminal)
    double precision:: cut_high(dim_cuth)   ! different difficulties to repeat the year in high school
    double precision :: cut_prim
    double precision:: cut_public(2)        ! different difficulties to repeat the year in public schools
    integer:: HScapped !=1 if high school tuition is subsidized so that tuition is capped
    double precision:: HSpriceCap !the upper bound on high school tuition for the students : let's use the highest public high school price or some low voucher private price. 
    double precision:: paraS(0:1) !parameter governing s0
    double precision:: lowGPAcost
    ! Inperpolation DPP    
    double precision, allocatable::Beta_DPP(:,:,:,:,:,:) ! Beta_DPP(dim_betas,bigS2,bigT-1,SizeDimXX,Ntype,Nloc)
       
! simulation
    integer,parameter:: seed=100 !random seed
    double precision,allocatable:: shock_gpa(:,:,:) !normal distributed: gpa shock(time, copy, id)
    double precision,allocatable:: shock_prog(:,:,:) !uniform distributed: whether one will be retained, given the calculated prob of retain Pr(retain|gpa,type)
    double precision,allocatable:: shock_choice(:,:,:) !uniform: which choice is the best, given the choice probabilities as the logit form. 
    double precision, allocatable:: shock_arrest(:,:,:) !uniform
    double precision,allocatable:: shock_jail(:,:,:) !uniform
    double precision,allocatable:: shock_transit(:,:) !uniform
    double precision,allocatable:: shock_abi(:,:)    !normal: the shock we use to assign a true ability level to each copy of the student, which is unknown to the student, but we use it to simulate the GPA.

! Estimation
    integer,parameter::tnp=123 !total number of possible parameters to be included
   

! Data moments
    double precision:: Medw3_5, Medw3_10, P75w6_5, P75w6_10, Medgpa, MedX3  
	
! Shocks interpolation method
    double precision,allocatable:: Aux_Ea_grid(:,:)
    double precision,allocatable:: Aux_Va_grid(:,:)
    double precision,allocatable:: Aux_Jgrid1(:,:)
    double precision,allocatable:: Aux_Jgrid2(:,:)
    double precision,allocatable:: Aux_Ggrid(:,:)
    double precision,allocatable:: Aux_Progrid(:,:)
    double precision,allocatable:: Aux_Egrid(:,:)
    double precision,allocatable:: Aux_Test_grid(:,:)
    double precision,allocatable:: Aux_Teacher_grid(:,:)
    double precision,allocatable:: Aux_Cut_grid(:,:)
    double precision,allocatable:: Aux_Tuition_grid(:,:)
	
	double precision,allocatable:: Aux_gpa_sim(:,:)
   

!Time counter
INTEGER  :: ctr1
INTEGER :: ClockStartAll_Total, ClockEndAll_Total, ClockRate_Total
INTEGER :: ClockStartAll_Type, ClockEndAll_Type, ClockRate_Type

!set random number seed
integer,parameter:: i_seed(12)= (/ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 10 /)
INTEGER :: i_size


end module variables

!****************CONVENTION***************
  !  x1 parental education (3 types: low (1), middle (2), high (3))
  !  x2=I(participation in social program)
  !  x3= family income: continous
  !  x4=i(private health insurance)
  !  x5: parent contract (3 types)
  !  x6=I(attened pre school)
  !  x7= family size: continous (we will use it in a discrete fashion, e.g., I(size>4))
    
  !  w1=0.0,1.0 (we only have public and voucher-private, 1.0: public): not interpolate
  !  w2=school SES categories (3): not interpolate
  !  w3= test score at school level 0. continuous: interpolate
  !  w4=fraction of experienced teachers 0. OR: average teacher experience >=0. continuous: interpolate.
  !  w5=cutoff for retention (this will be another state variable) continuous to be interpolated! 
  !  w6=tuition>=0. continuous: interpolate
            
! Choices: (e=0,d=0): 1, (e=0,d=1): 2, (e=1,d=0): 3, (e=1,d=1): 4. 

!DataStudent:

!ID; parental education; I(single parent), family income; i(private health insurance); parent contract; I(attened pre school); family size; School ID; Location

!Datasch (2-7 primary schools, 8-13 secondary school):

!School ID; I(public school); SES; tuition/1000; average test score (standardized); fraction of experienced teachers; cutoff for retention; I(public school); SES; tuition/1000; average test score (standardized); fraction of experienced teachers; cutoff for retention; offers high schools
